@@ -6,6 +6,17 @@ from django.utils.translation import gettext_lazy as _
 from formset.widgets import UploadedFileInput
 
 
+class CheckboxInputMixin:
+    """
+    This hack is required for adding the field's label to the rendering context.
+    This is to make the single checkbox to be rendered with its label after the input field.
+    """
+    def get_context(self, name, value, attrs):
+        context = super().get_context(name, value, attrs)
+        context['label'] = self.label
+        return context
+
+
 class BoundField(boundfield.BoundField):
     @property
     def errors(self):
@@ -14,12 +25,29 @@ class BoundField(boundfield.BoundField):
         return errors
 
     def as_widget(self, widget=None, attrs=None, only_initial=False):
-        if widget is None:
-            widget = self.form.get_widget(self.field)
-        return super().as_widget(widget, attrs, only_initial)
+        widget = widget or self.field.widget
+        if self.widget_type == 'checkbox':
+            widget.__class__ = type(widget.__class__.__name__, (CheckboxInputMixin, widget.__class__), {'label': self.label})
+        if self.field.localize:
+            widget.is_localized = True
+        attrs = attrs or {}
+        attrs = self.build_widget_attrs(attrs, widget)
+        if self.auto_id and 'id' not in widget.attrs:
+            attrs.setdefault('id', self.html_initial_id if only_initial else self.auto_id)
+        return widget.render(
+            name=self.name,  # `self.html_name` contains the full path and hence doesn't work with form collections
+            value=self.value(),
+            attrs=attrs,
+            renderer=self.form.renderer,
+        )
 
-    def label_tag(self, contents=None, attrs=None, label_suffix=None):
-        return self.form.render_label(self, contents, attrs, label_suffix)
+    def build_widget_attrs(self, attrs, widget=None):
+        attrs = super().build_widget_attrs(attrs, widget)
+        if self.form.form_id:
+            attrs['form'] = self.form.form_id
+        if hasattr(self.field, 'regex'):
+            attrs['pattern'] = self.field.regex.pattern
+        return attrs
 
     def css_classes(self, extra_classes=None):
         """
@@ -34,8 +62,8 @@ class BoundField(boundfield.BoundField):
             else:
                 extra_classes.add('dj-required')
 
-        # field_css_classes is an optional member of a Form optimized for django-formset
-        field_css_classes = getattr(self.form, 'field_css_classes', None)
+        # field_css_classes is an optional member of a FormRenderer optimized for django-formset
+        field_css_classes = getattr(self.form.renderer, 'field_css_classes', None)
         if hasattr(field_css_classes, 'split'):
             extra_classes.update(field_css_classes.split())
         elif isinstance(field_css_classes, (list, tuple)):
@@ -46,7 +74,7 @@ class BoundField(boundfield.BoundField):
     def widget_type(self):
         return super().widget_type
 
-    @property
+    @cached_property
     def auto_id(self):
         """
         Since we can have many forms with a different name each, prefix the id with the form name
@@ -61,9 +89,6 @@ class BoundField(boundfield.BoundField):
             return self.html_name
         return ''
 
-    def build_widget_attrs(self, attrs, widget=None):
-        return self.form.get_widget_attrs(self, attrs, widget)
-
     def _get_client_messages(self):
         """
         Extract server validation error messages to be rendered by the client side.
@@ -74,7 +99,10 @@ class BoundField(boundfield.BoundField):
             if self.widget_type == 'checkboxselectmultiple':
                 client_messages['custom_error'] = _("At least one checkbox must be selected.")
             elif 'required' in server_messages:
-                client_messages['value_missing'] = server_messages['required']
+                if self.widget_type == 'selectize':
+                    client_messages['custom_error'] = server_messages['required']
+                else:
+                    client_messages['value_missing'] = server_messages['required']
         if 'invalid' in server_messages:
             client_messages['type_mismatch'] = client_messages['pattern_mismatch'] = server_messages['invalid']
         elif 'invalid_choice' in server_messages:
